#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = "Dr Mohamed Salaheddine Talamali - University College London (UCL)"

import numpy as np
import math
import matplotlib.pyplot as plt
import csv
from matplotlib.backends.backend_pdf import PdfPages
import zipfile
import os
from scipy import stats
import scipy as sp
from matplotlib import rcParams
from mpl_toolkits.axes_grid1 import make_axes_locatable
import matplotlib.gridspec as gridspec

# Legend personalization
import matplotlib.patches as mpatches
import matplotlib.lines as mlines
import warnings

def binomial_confidence_interval(data,number,confidence=0.95):
	p=data
	z=1-confidence/2.0
	conf=z*math.sqrt(p*(1-p)/float(number))
	min=p-conf
	max=p+conf
	return max,min 

def mean_confidence_interval(data, confidence=0.95):
    a = 1.0*np.array(data)
    n = len(a)
    m, se = np.mean(a), sp.stats.sem(a)
    conf = se * sp.stats.t._ppf((1+confidence)/2., n-1)
    return m, m+conf, m-conf

def movingaverage(values,window):
	weigths = np.repeat(1.0, window)/window
	smas = np.convolve(values, weigths,'valid')
	extended_smas = [0] * (window-1)
	extended_smas.extend(smas)
	return extended_smas # as a numpy array

# Number of runs
numberofruns=6

# Experiment timestep (in seconds) 
sampling_time = 0.5

# Data plotting steps (in experiment timestep)
step=10

# Fontsize
fontsize=48

# Quorum threshold
Q = 0.8

# raw data file
data_archive = zipfile.ZipFile('raw_data.zip', 'r')

# total number of options
n=3

# number of robots
N = 50

# communication range
commrng=0.1

# robot's field of view 
opsize=0.2

# Types of communication ranges
communication_types=["local_communication","global_communication"]

fig=plt.figure(figsize=(15,7.5))
plt.rc('text', usetex=True)
rcParams['font.family'] = 'sans-serif'
rcParams['font.sans-serif'] = ['Myriad Pro']

gs = gridspec.GridSpec(1, 2, width_ratios=[1, 2]) 


for communication_type in communication_types:

	if(communication_type=="local_communication"):
		numberoftimesteps=1200
		ax1 = plt.subplot(gs[0])
		x_max=40.2
		x_min=-0.2
		y_max=1.02
		y_min=-0.02
		ax1.set_xticks(np.arange(0,41,10))
		ax1.set_xticklabels(np.arange(0,int(21),5),fontsize=fontsize-6)
		ax1.set_title(r"\textbf{(E)} "+"  Adaptation: "+"$100\%$",fontsize=36)
		ax1.axhline(y=0.8,linewidth=3, color='black',linestyle='--')

	if(communication_type=="global_communication"):
		numberoftimesteps=2400
		ax1 = plt.subplot(gs[1])
		x_max=40.2
		x_min=-0.2
		y_max=1.02
		y_min=-0.02
		ax1.get_yaxis().set_visible(False)

		ax1.set_xticks(np.arange(0,41,5))
		ax1.set_xticklabels(np.arange(0,int(41),5),fontsize=fontsize-6)

		ax1.set_title(r"\textbf{(F)} "+"  Adaptation: "+"$33\%$",fontsize=36)
		ax1.axhline(y=0.8,linewidth=3, color='black',linestyle='--')


	filename_prefix="timePop_"+communication_type

	dataarray=np.zeros((numberofruns,int(numberoftimesteps/sampling_time)+1,n+1))

	file_to_save="post_processed_data/"+communication_type+".npy"

	if not os.path.exists(file_to_save):
		print("Processing row date...")
		for i in np.arange(1,numberofruns+1):
			filename=communication_type+"/"+filename_prefix+"_"+str(i)+".txt"
			print("run=",i)
			csvfile=data_archive.open(filename)
			data=np.loadtxt(csvfile, delimiter='\t', usecols = (0,1,2,3,4,5,6,7,8))
			data=np.array(data)

			timestep=0
			
			dataarray[i-1,timestep,0] = timestep*sampling_time
			dataarray[i-1,timestep,1] = 0
			dataarray[i-1,timestep,2] = 0
			dataarray[i-1,timestep,3] = 50.0

			timestep=timestep+1

			for j in np.arange(2,data.shape[0]+1):

				while(data[j,0]>timestep*sampling_time):
					if(timestep>int(numberoftimesteps/sampling_time)):
						break
					dataarray[i-1,timestep,0] = timestep*sampling_time
					dataarray[i-1,timestep,1] = dataarray[i-1,timestep-1,1]
					dataarray[i-1,timestep,2] = dataarray[i-1,timestep-1,2]
					dataarray[i-1,timestep,3] = dataarray[i-1,timestep-1,3]
					timestep=timestep+1
					if(timestep>int(numberoftimesteps/sampling_time)):
						break
				if(timestep>int(numberoftimesteps/sampling_time)):
						break

				dataarray[i-1,timestep,0] = timestep*sampling_time
				dataarray[i-1,timestep,1] = data[j,2]
				dataarray[i-1,timestep,2] = data[j,3]
				dataarray[i-1,timestep,3] = data[j,4]
				timestep=timestep+1

		np.save(file_to_save,dataarray)

	else:
		print("Opening data file ...")
		dataarray=np.load(file_to_save)



	OP1_AVG=[]
	OP1_MAX=[]
	OP1_MIN=[]

	OP2_AVG=[]
	OP2_MAX=[]
	OP2_MIN=[]

	OP3_AVG=[]
	OP3_MAX=[]
	OP3_MIN=[]


	TIME=[]



	plot_linewidth=5
	lines_linewidth=3

	linewidth_plots=3
	alpha=0.5

	for timestep in np.arange(0,dataarray.shape[1],8):

		TIME.append(timestep*sampling_time/60.0)

		OP3=dataarray[:,timestep,3]/N
		plt.scatter(np.full((1, len(OP3)), timestep/60.0), OP3,c='blue',marker='s',alpha=alpha)


	for timestep in np.arange(0,dataarray.shape[1],4):

		TIME.append(timestep*sampling_time/60.0)

		OP1=dataarray[:,timestep,1]/N
		plt.scatter(np.full((1, len(OP1)), timestep/60.0), OP1,c='red',marker='o',alpha=alpha)

	plt.xlim((x_min,x_max))
	plt.ylim((y_min,y_max))

	ax1.set_ylabel("Population $(\%)$",fontsize=fontsize)
	ax1.set_yticks(np.arange(0,1.01,0.2))
	ax1.set_yticklabels(("0","20","40","60","80","100"),fontsize=fontsize)

	ax1.set_xlabel("Time $(min)$",fontsize=fontsize)

	for axis in ['top','bottom','left','right']:
		ax1.spines[axis].set_linewidth(4)

	ax1.tick_params('both', length=8, width=4, which='major')
	ax1.tick_params('both', length=6, width=2, which='minor')


with warnings.catch_warnings():
	warnings.simplefilter("ignore", category=RuntimeWarning)
	# Colors legend
	A1_line=mlines.Line2D([], [],linestyle='',color='r', linewidth=4 ,label="Best site $\mathcal{T}_x$",marker='o',markersize=8)
	A2_line=mlines.Line2D([], [],linestyle='' ,color='b', linewidth=4 ,label="Second best site $\mathcal{T}_y$",marker='s',markersize=8)

	l1=plt.figlegend(handles=[A1_line,A2_line],edgecolor='black',fontsize=18,handlelength=1,loc=9,bbox_to_anchor=(0.3,0.5, 1., .102),numpoints=1,frameon=True, shadow=False, fancybox=True,ncol=1,columnspacing=0.8,handletextpad=0.4,labelspacing=0.2)

plt.tight_layout()
pp = PdfPages("Fig6_E_and_F_no_overlay.pdf")
pp.savefig(bbox_inches='tight')
pp.close()
plt.clf()